@@ -18,93 +18,147 @@ class _IPv4Constants:
     Excluded network list is updated to comply with current IANA list of
     private and reserved networks.
     """
+
     _network_classes: Dict[str, IPv4Network] = {
-        'a': ip_network('0.0.0.0/1'),
-        'b': ip_network('128.0.0.0/2'),
-        'c': ip_network('192.0.0.0/3'),
+        "a": ip_network("0.0.0.0/1"),
+        "b": ip_network("128.0.0.0/2"),
+        "c": ip_network("192.0.0.0/3"),
     }
 
     # Three common private networks from class A, B and CIDR
     # to generate private addresses from.
     _private_networks: List[IPv4Network] = [
-        ip_network('10.0.0.0/8'),
-        ip_network('172.16.0.0/12'),
-        ip_network('192.168.0.0/16'),
+        ip_network("10.0.0.0/8"),
+        ip_network("172.16.0.0/12"),
+        ip_network("192.168.0.0/16"),
     ]
 
     # List of networks from which IP addresses will never be generated,
     # includes other private IANA and reserved networks from
     # ttps://www.iana.org/assignments/iana-ipv4-special-registry/iana-ipv4-special-registry.xhtml
     _excluded_networks: List[IPv4Network] = [
-        ip_network('0.0.0.0/8'),
-        ip_network('100.64.0.0/10'),
-        ip_network('127.0.0.0/8'),  # loopback network
-        ip_network('169.254.0.0/16'),  # linklocal network
-        ip_network('192.0.0.0/24'),
-        ip_network('192.0.2.0/24'),
-        ip_network('192.31.196.0/24'),
-        ip_network('192.52.193.0/24'),
-        ip_network('192.88.99.0/24'),
-        ip_network('192.175.48.0/24'),
-        ip_network('198.18.0.0/15'),
-        ip_network('198.51.100.0/24'),
-        ip_network('203.0.113.0/24'),
-        ip_network('224.0.0.0/4'),  # multicast network
-        ip_network('240.0.0.0/4'),
-        ip_network('255.255.255.255/32'),
+        ip_network("0.0.0.0/8"),
+        ip_network("100.64.0.0/10"),
+        ip_network("127.0.0.0/8"),  # loopback network
+        ip_network("169.254.0.0/16"),  # linklocal network
+        ip_network("192.0.0.0/24"),
+        ip_network("192.0.2.0/24"),
+        ip_network("192.31.196.0/24"),
+        ip_network("192.52.193.0/24"),
+        ip_network("192.88.99.0/24"),
+        ip_network("192.175.48.0/24"),
+        ip_network("198.18.0.0/15"),
+        ip_network("198.51.100.0/24"),
+        ip_network("203.0.113.0/24"),
+        ip_network("224.0.0.0/4"),  # multicast network
+        ip_network("240.0.0.0/4"),
+        ip_network("255.255.255.255/32"),
     ]
 
 
 class Provider(BaseProvider):
-    safe_domain_names: ElementsType = ('example.org', 'example.com', 'example.net')
-    free_email_domains: ElementsType = ('gmail.com', 'yahoo.com', 'hotmail.com')
+    safe_domain_names: ElementsType = ("example.org", "example.com", "example.net")
+    free_email_domains: ElementsType = ("gmail.com", "yahoo.com", "hotmail.com")
     tlds: ElementsType = (
-        'com', 'com', 'com', 'com', 'com', 'com', 'biz', 'info', 'net', 'org',
+        "com",
+        "com",
+        "com",
+        "com",
+        "com",
+        "com",
+        "biz",
+        "info",
+        "net",
+        "org",
+    )
+    hostname_prefixes: ElementsType = (
+        "db",
+        "srv",
+        "desktop",
+        "laptop",
+        "lt",
+        "email",
+        "web",
     )
-    hostname_prefixes: ElementsType = ('db', 'srv', 'desktop', 'laptop', 'lt', 'email', 'web')
     uri_pages: ElementsType = (
-        'index', 'home', 'search', 'main', 'post', 'homepage', 'category',
-        'register', 'login', 'faq', 'about', 'terms', 'privacy', 'author',
+        "index",
+        "home",
+        "search",
+        "main",
+        "post",
+        "homepage",
+        "category",
+        "register",
+        "login",
+        "faq",
+        "about",
+        "terms",
+        "privacy",
+        "author",
     )
     uri_paths: ElementsType = (
-        'app', 'main', 'wp-content', 'search', 'category', 'tag', 'categories',
-        'tags', 'blog', 'posts', 'list', 'explore',
+        "app",
+        "main",
+        "wp-content",
+        "search",
+        "category",
+        "tag",
+        "categories",
+        "tags",
+        "blog",
+        "posts",
+        "list",
+        "explore",
     )
     uri_extensions: ElementsType = (
-        '.html', '.html', '.html', '.htm', '.htm', '.php', '.php', '.jsp',
-        '.asp',
+        ".html",
+        ".html",
+        ".html",
+        ".htm",
+        ".htm",
+        ".php",
+        ".php",
+        ".jsp",
+        ".asp",
     )
     http_methods: ElementsType = (
-        'GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'CONNECT', 'OPTIONS', 'TRACE',
-        'PATCH',
+        "GET",
+        "HEAD",
+        "POST",
+        "PUT",
+        "DELETE",
+        "CONNECT",
+        "OPTIONS",
+        "TRACE",
+        "PATCH",
     )
 
     user_name_formats: ElementsType = (
-        '{{last_name}}.{{first_name}}',
-        '{{first_name}}.{{last_name}}',
-        '{{first_name}}##',
-        '?{{last_name}}',
+        "{{last_name}}.{{first_name}}",
+        "{{first_name}}.{{last_name}}",
+        "{{first_name}}##",
+        "?{{last_name}}",
     )
     email_formats: ElementsType = (
-        '{{user_name}}@{{domain_name}}',
-        '{{user_name}}@{{free_email_domain}}',
+        "{{user_name}}@{{domain_name}}",
+        "{{user_name}}@{{free_email_domain}}",
     )
     url_formats: ElementsType = (
-        'www.{{domain_name}}/',
-        '{{domain_name}}/',
+        "www.{{domain_name}}/",
+        "{{domain_name}}/",
     )
     uri_formats: ElementsType = (
-        '{{url}}',
-        '{{url}}{{uri_page}}/',
-        '{{url}}{{uri_page}}{{uri_extension}}',
-        '{{url}}{{uri_path}}/{{uri_page}}/',
-        '{{url}}{{uri_path}}/{{uri_page}}{{uri_extension}}',
+        "{{url}}",
+        "{{url}}{{uri_page}}/",
+        "{{url}}{{uri_page}}{{uri_extension}}",
+        "{{url}}{{uri_path}}/{{uri_page}}/",
+        "{{url}}{{uri_path}}/{{uri_page}}{{uri_extension}}",
     )
     image_placeholder_services: ElementsType = (
-        'https://www.lorempixel.com/{width}/{height}',
-        'https://dummyimage.com/{width}x{height}',
-        'https://placekitten.com/{width}/{height}',
-        'https://placeimg.com/{width}/{height}/any',
+        "https://www.lorempixel.com/{width}/{height}",
+        "https://dummyimage.com/{width}x{height}",
+        "https://placekitten.com/{width}/{height}",
+        "https://placeimg.com/{width}/{height}/any",
     )
 
     replacements: Tuple[Tuple[str, str], ...] = ()
@@ -119,9 +173,9 @@ class Provider(BaseProvider):
     @lowercase
     def email(self, safe: bool = True, domain: Optional[str] = None) -> str:
         if domain:
-            email = f'{self.user_name()}@{domain}'
+            email = f"{self.user_name()}@{domain}"
         elif safe:
-            email = f'{self.user_name()}@{self.safe_domain_name()}'
+            email = f"{self.user_name()}@{self.safe_domain_name()}"
         else:
             pattern: str = self.random_element(self.email_formats)
             email = "".join(self.generator.parse(pattern).split(" "))
@@ -133,15 +187,15 @@ class Provider(BaseProvider):
 
     @lowercase
     def safe_email(self) -> str:
-        return self.user_name() + '@' + self.safe_domain_name()
+        return self.user_name() + "@" + self.safe_domain_name()
 
     @lowercase
     def free_email(self) -> str:
-        return self.user_name() + '@' + self.free_email_domain()
+        return self.user_name() + "@" + self.free_email_domain()
 
     @lowercase
     def company_email(self) -> str:
-        return self.user_name() + '@' + self.domain_name()
+        return self.user_name() + "@" + self.domain_name()
 
     @lowercase
     def free_email_domain(self) -> str:
@@ -156,18 +210,18 @@ class Provider(BaseProvider):
 
     @lowercase
     def ascii_safe_email(self) -> str:
-        return self._to_ascii(self.user_name() + '@' + self.safe_domain_name())
+        return self._to_ascii(self.user_name() + "@" + self.safe_domain_name())
 
     @lowercase
     def ascii_free_email(self) -> str:
         return self._to_ascii(
-            self.user_name() + '@' + self.free_email_domain(),
+            self.user_name() + "@" + self.free_email_domain(),
         )
 
     @lowercase
     def ascii_company_email(self) -> str:
         return self._to_ascii(
-            self.user_name() + '@' + self.domain_name(),
+            self.user_name() + "@" + self.domain_name(),
         )
 
     @slugify_unicode
@@ -188,9 +242,10 @@ class Provider(BaseProvider):
         web-12.williamson-hopkins.jackson.com
         """
         hostname_prefix: str = self.random_element(self.hostname_prefixes)
-        hostname_prefix_first_level: str = hostname_prefix + '-' + self.numerify('##')
-        return hostname_prefix_first_level if levels < 1 \
-            else hostname_prefix_first_level + '.' + self.domain_name(levels)
+        hostname_prefix_first_level: str = hostname_prefix + "-" + self.numerify("##")
+        return (
+            hostname_prefix_first_level if levels < 1 else hostname_prefix_first_level + "." + self.domain_name(levels)
+        )
 
     @lowercase
     def domain_name(self, levels: int = 1) -> str:
@@ -206,22 +261,24 @@ class Provider(BaseProvider):
         if levels < 1:
             raise ValueError("levels must be greater than or equal to 1")
         if levels == 1:
-            return self.domain_word() + '.' + self.tld()
-        return self.domain_word() + '.' + self.domain_name(levels - 1)
+            return self.domain_word() + "." + self.tld()
+        return self.domain_word() + "." + self.domain_name(levels - 1)
 
     @lowercase
     @slugify_unicode
     def domain_word(self) -> str:
-        company: str = self.generator.format('company')
-        company_elements: List[str] = company.split(' ')
+        company: str = self.generator.format("company")
+        company_elements: List[str] = company.split(" ")
         return self._to_ascii(company_elements.pop(0))
 
-    def dga(self,
-            year: Optional[int] = None,
-            month: Optional[int] = None,
-            day: Optional[int] = None,
-            tld: Optional[str] = None,
-            length: Optional[int] = None) -> str:
+    def dga(
+        self,
+        year: Optional[int] = None,
+        month: Optional[int] = None,
+        day: Optional[int] = None,
+        tld: Optional[str] = None,
+        length: Optional[int] = None,
+    ) -> str:
         """Generates a domain name by given date
         https://en.wikipedia.org/wiki/Domain_generation_algorithm
 
@@ -233,7 +290,7 @@ class Provider(BaseProvider):
         :rtype: str
         """
 
-        domain = ''
+        domain = ""
         year = year or self.random_int(min=1, max=9999)
         month = month or self.random_int(min=1, max=12)
         day = day or self.random_int(min=1, max=30)
@@ -246,7 +303,7 @@ class Provider(BaseProvider):
             day = ((day ^ (day << 13)) >> 19) ^ ((day & 0xFFFFFFFE) << 12)
             domain += chr(((year ^ month ^ day) % 25) + 97)
 
-        return domain + '.' + tld
+        return domain + "." + tld
 
     def tld(self) -> str:
         return self.random_element(self.tlds)
@@ -269,14 +326,13 @@ class Provider(BaseProvider):
         :returns: a random url string.
         """
         if schemes is None:
-            schemes = ['http', 'https']
+            schemes = ["http", "https"]
 
         pattern: str = f'{self.random_element(schemes) if schemes else ""}://{self.random_element(self.url_formats)}'
 
         return self.generator.parse(pattern)
 
-    def _get_all_networks_and_weights(self,
-                                      address_class: Optional[str] = None) -> Tuple[List[IPv4Network], List[int]]:
+    def _get_all_networks_and_weights(self, address_class: Optional[str] = None) -> Tuple[List[IPv4Network], List[int]]:
         """
         Produces a 2-tuple of valid IPv4 networks and corresponding relative weights
 
@@ -284,14 +340,14 @@ class Provider(BaseProvider):
         """
         # If `address_class` has an unexpected value, use the whole IPv4 pool
         if address_class in _IPv4Constants._network_classes.keys():
-            networks_attr = f'_cached_all_class_{address_class}_networks'
+            networks_attr = f"_cached_all_class_{address_class}_networks"
             all_networks = [_IPv4Constants._network_classes[address_class]]  # type: ignore
         else:
-            networks_attr = '_cached_all_networks'
-            all_networks = [ip_network('0.0.0.0/0')]
+            networks_attr = "_cached_all_networks"
+            all_networks = [ip_network("0.0.0.0/0")]
 
         # Return cached network and weight data if available
-        weights_attr = f'{networks_attr}_weights'
+        weights_attr = f"{networks_attr}_weights"
         if hasattr(self, networks_attr) and hasattr(self, weights_attr):
             return getattr(self, networks_attr), getattr(self, weights_attr)
 
@@ -309,9 +365,10 @@ class Provider(BaseProvider):
         setattr(self, weights_attr, weights)
         return all_networks, weights
 
-    def _get_private_networks_and_weights(self,
-                                          address_class: Optional[str] = None,
-                                          ) -> Tuple[List[IPv4Network], List[int]]:
+    def _get_private_networks_and_weights(
+        self,
+        address_class: Optional[str] = None,
+    ) -> Tuple[List[IPv4Network], List[int]]:
         """
         Produces an OrderedDict of valid private IPv4 networks and corresponding relative weights
 
@@ -322,17 +379,14 @@ class Provider(BaseProvider):
             address_class = self.ipv4_network_class()
 
         # Return cached network and weight data if available for a specific address class
-        networks_attr = f'_cached_private_class_{address_class}_networks'
-        weights_attr = f'{networks_attr}_weights'
+        networks_attr = f"_cached_private_class_{address_class}_networks"
+        weights_attr = f"{networks_attr}_weights"
         if hasattr(self, networks_attr) and hasattr(self, weights_attr):
             return getattr(self, networks_attr), getattr(self, weights_attr)
 
         # Otherwise, compute for list of private networks (excluding special networks)
         supernet = _IPv4Constants._network_classes[address_class]
-        private_networks = [
-            subnet for subnet in _IPv4Constants._private_networks
-            if subnet.overlaps(supernet)
-        ]
+        private_networks = [subnet for subnet in _IPv4Constants._private_networks if subnet.overlaps(supernet)]
         private_networks = self._exclude_ipv4_networks(
             private_networks,
             _IPv4Constants._excluded_networks,
@@ -346,9 +400,10 @@ class Provider(BaseProvider):
         setattr(self, weights_attr, weights)
         return private_networks, weights
 
-    def _get_public_networks_and_weights(self,
-                                         address_class: Optional[str] = None,
-                                         ) -> Tuple[List[IPv4Network], List[int]]:
+    def _get_public_networks_and_weights(
+        self,
+        address_class: Optional[str] = None,
+    ) -> Tuple[List[IPv4Network], List[int]]:
         """
         Produces a 2-tuple of valid public IPv4 networks and corresponding relative weights
 
@@ -359,8 +414,8 @@ class Provider(BaseProvider):
             address_class = self.ipv4_network_class()
 
         # Return cached network and weight data if available for a specific address class
-        networks_attr = f'_cached_public_class_{address_class}_networks'
-        weights_attr = f'{networks_attr}_weights'
+        networks_attr = f"_cached_public_class_{address_class}_networks"
+        weights_attr = f"{networks_attr}_weights"
         if hasattr(self, networks_attr) and hasattr(self, weights_attr):
             return getattr(self, networks_attr), getattr(self, weights_attr)
 
@@ -368,8 +423,7 @@ class Provider(BaseProvider):
         public_networks = [_IPv4Constants._network_classes[address_class]]  # type: ignore
         public_networks = self._exclude_ipv4_networks(
             public_networks,
-            _IPv4Constants._private_networks +
-            _IPv4Constants._excluded_networks,
+            _IPv4Constants._private_networks + _IPv4Constants._excluded_networks,
         )
 
         # Then compute for list of corresponding relative weights
@@ -380,10 +434,12 @@ class Provider(BaseProvider):
         setattr(self, weights_attr, weights)
         return public_networks, weights
 
-    def _random_ipv4_address_from_subnets(self,
-                                          subnets: List[IPv4Network],
-                                          weights: Optional[List[int]] = None,
-                                          network: bool = False) -> str:
+    def _random_ipv4_address_from_subnets(
+        self,
+        subnets: List[IPv4Network],
+        weights: Optional[List[int]] = None,
+        network: bool = False,
+    ) -> str:
         """
         Produces a random IPv4 address or network with a valid CIDR
         from within the given subnets using a distribution described
@@ -395,35 +451,46 @@ class Provider(BaseProvider):
         :return:
         """
         if not subnets:
-            raise ValueError('No subnets to choose from')
+            raise ValueError("No subnets to choose from")
 
         # If the weights argument has an invalid value, default to equal distribution
-        if (isinstance(weights, list) and len(subnets) == len(weights) and
-                all(isinstance(w, (float, int)) for w in weights)):
-            subnet = choices_distribution(subnets,
-                                          [float(w) for w in weights],
-                                          random=self.generator.random, length=1)[0]
+        if (
+            isinstance(weights, list)
+            and len(subnets) == len(weights)
+            and all(isinstance(w, (float, int)) for w in weights)
+        ):
+            subnet = choices_distribution(
+                subnets,
+                [float(w) for w in weights],
+                random=self.generator.random,
+                length=1,
+            )[0]
         else:
             subnet = self.generator.random.choice(subnets)
 
         address = str(
-            subnet[self.generator.random.randint(
-                0, subnet.num_addresses - 1,
-            )],
+            subnet[
+                self.generator.random.randint(
+                    0,
+                    subnet.num_addresses - 1,
+                )
+            ],
         )
 
         if network:
-            address += '/' + str(self.generator.random.randint(
-                subnet.prefixlen,
-                subnet.max_prefixlen,
-            ))
+            address += "/" + str(
+                self.generator.random.randint(
+                    subnet.prefixlen,
+                    subnet.max_prefixlen,
+                )
+            )
             address = str(ip_network(address, strict=False))
 
         return address
 
-    def _exclude_ipv4_networks(self,
-                               networks: List[IPv4Network],
-                               networks_to_exclude: List[IPv4Network]) -> List[IPv4Network]:
+    def _exclude_ipv4_networks(
+        self, networks: List[IPv4Network], networks_to_exclude: List[IPv4Network]
+    ) -> List[IPv4Network]:
         """
         Exclude the list of networks from another list of networks
         and return a flat list of new networks.
@@ -434,6 +501,7 @@ class Provider(BaseProvider):
         """
         networks_to_exclude.sort(key=lambda x: x.prefixlen)
         for network_to_exclude in networks_to_exclude:
+
             def _exclude_ipv4_network(network):
                 """
                 Exclude a single network from another single network
@@ -469,9 +537,14 @@ class Provider(BaseProvider):
 
         :returns: IPv4 network class
         """
-        return self.random_element('abc')
+        return self.random_element("abc")
 
-    def ipv4(self, network: bool = False, address_class: Optional[str] = None, private: Optional[str] = None) -> str:
+    def ipv4(
+        self,
+        network: bool = False,
+        address_class: Optional[str] = None,
+        private: Optional[str] = None,
+    ) -> str:
         """
         Returns a random IPv4 address or network with a valid CIDR.
 
@@ -481,11 +554,9 @@ class Provider(BaseProvider):
         :returns: IPv4
         """
         if private is True:
-            return self.ipv4_private(address_class=address_class,
-                                     network=network)
+            return self.ipv4_private(address_class=address_class, network=network)
         elif private is False:
-            return self.ipv4_public(address_class=address_class,
-                                    network=network)
+            return self.ipv4_public(address_class=address_class, network=network)
         else:
             all_networks, weights = self._get_all_networks_and_weights(address_class=address_class)
             return self._random_ipv4_address_from_subnets(all_networks, weights=weights, network=network)
@@ -514,15 +585,14 @@ class Provider(BaseProvider):
 
     def ipv6(self, network: bool = False) -> str:
         """Produce a random IPv6 address or network with a valid CIDR"""
-        address = str(ip_address(self.generator.random.randint(
-            2 ** IPV4LENGTH, (2 ** IPV6LENGTH) - 1)))
+        address = str(ip_address(self.generator.random.randint(2 ** IPV4LENGTH, (2 ** IPV6LENGTH) - 1)))
         if network:
-            address += '/' + str(self.generator.random.randint(0, IPV6LENGTH))
+            address += "/" + str(self.generator.random.randint(0, IPV6LENGTH))
             address = str(ip_network(address, strict=False))
         return address
 
     def mac_address(self) -> str:
-        mac = [self.generator.random.randint(0x00, 0xff) for _ in range(0, 6)]
+        mac = [self.generator.random.randint(0x00, 0xFF) for _ in range(0, 6)]
         return ":".join(map(lambda x: "%02x" % x, mac))
 
     def port_number(self, is_system: bool = False, is_user: bool = False, is_dynamic: bool = False) -> int:
@@ -593,11 +663,11 @@ class Provider(BaseProvider):
         :rtype: str
         """
 
-        lex = '?' * self.random_int(min=2, max=4)
-        num = '%' * self.random_int(min=1, max=5)
-        return self.bothify(f'ORG-{lex}{num}-RIPE').upper()
+        lex = "?" * self.random_int(min=2, max=4)
+        num = "%" * self.random_int(min=1, max=5)
+        return self.bothify(f"ORG-{lex}{num}-RIPE").upper()
 
-    def nic_handle(self, suffix: str = 'FAKE') -> str:
+    def nic_handle(self, suffix: str = "FAKE") -> str:
         """Returns NIC Handle ID
         https://www.apnic.net/manage-ip/using-whois/guide/person/
 
@@ -605,13 +675,13 @@ class Provider(BaseProvider):
         """
 
         if len(suffix) < 2:
-            raise ValueError('suffix length must be greater than or equal to 2')
+            raise ValueError("suffix length must be greater than or equal to 2")
 
-        lex = '?' * self.random_int(min=2, max=4)
-        num = '%' * self.random_int(min=1, max=5)
-        return self.bothify(f'{lex}{num}-{suffix}').upper()
+        lex = "?" * self.random_int(min=2, max=4)
+        num = "%" * self.random_int(min=1, max=5)
+        return self.bothify(f"{lex}{num}-{suffix}").upper()
 
-    def nic_handles(self, count: int = 1, suffix: str = '????') -> List[str]:
+    def nic_handles(self, count: int = 1, suffix: str = "????") -> List[str]:
         """Returns NIC Handle ID list
 
         :rtype: list[str]
