@@ -4,10 +4,10 @@ from .. import Provider as BaseProvider
 
 
 class Provider(BaseProvider):
-    INVALID_SSN_TYPE = 'INVALID_SSN'
-    SSN_TYPE = 'SSN'
-    ITIN_TYPE = 'ITIN'
-    EIN_TYPE = 'EIN'
+    INVALID_SSN_TYPE = "INVALID_SSN"
+    SSN_TYPE = "SSN"
+    ITIN_TYPE = "ITIN"
+    EIN_TYPE = "EIN"
 
     def itin(self) -> str:
         """Generate a random United States Individual Taxpayer Identification Number (ITIN).
@@ -28,19 +28,19 @@ class Provider(BaseProvider):
         # The group number must be between 70 and 99 inclusively but not 89 or 93
         group: int = self.random_element([x for x in range(70, 100) if x not in [89, 93]])
 
-        itin = f'{area:03d}-{group:02d}-{serial:04d}'
+        itin = f"{area:03d}-{group:02d}-{serial:04d}"
         return itin
 
     def ein(self) -> str:
         """Generate a random United States Employer Identification Number (EIN).
 
-         An United States An Employer Identification Number (EIN) is
-         also known as a Federal Tax Identification Number, and is
-         used to identify a business entity. EINs follow a format of a
-         two-digit prefix followed by a hyphen and a seven-digit sequence:
-         ##-######
+        An United States An Employer Identification Number (EIN) is
+        also known as a Federal Tax Identification Number, and is
+        used to identify a business entity. EINs follow a format of a
+        two-digit prefix followed by a hyphen and a seven-digit sequence:
+        ##-######
 
-         https://www.irs.gov/businesses/small-businesses-self-employed/employer-id-numbers
+        https://www.irs.gov/businesses/small-businesses-self-employed/employer-id-numbers
         """
 
         # Only certain EIN Prefix values are assigned:
@@ -48,98 +48,99 @@ class Provider(BaseProvider):
         # https://www.irs.gov/businesses/small-businesses-self-employed/how-eins-are-assigned-and-valid-ein-prefixes
 
         ein_prefix_choices: List[str] = [
-            '01',
-            '02',
-            '03',
-            '04',
-            '05',
-            '06',
-            '10',
-            '11',
-            '12',
-            '13',
-            '14',
-            '15',
-            '16',
-            '20',
-            '21',
-            '22',
-            '23',
-            '24',
-            '25',
-            '26',
-            '27',
-            '30',
-            '31',
-            '32',
-            '33',
-            '34',
-            '35',
-            '36',
-            '37',
-            '38',
-            '39',
-            '40',
-            '41',
-            '42',
-            '43',
-            '44',
-            '45',
-            '46',
-            '47',
-            '48',
-            '50',
-            '51',
-            '52',
-            '53',
-            '54',
-            '55',
-            '56',
-            '57',
-            '58',
-            '59',
-            '60',
-            '61',
-            '62',
-            '63',
-            '64',
-            '65',
-            '66',
-            '67',
-            '68',
-            '71',
-            '72',
-            '73',
-            '74',
-            '75',
-            '76',
-            '77',
-            '80',
-            '81',
-            '82',
-            '83',
-            '84',
-            '85',
-            '86',
-            '87',
-            '88',
-            '90',
-            '91',
-            '92',
-            '93',
-            '94',
-            '95',
-            '98',
-            '99']
+            "01",
+            "02",
+            "03",
+            "04",
+            "05",
+            "06",
+            "10",
+            "11",
+            "12",
+            "13",
+            "14",
+            "15",
+            "16",
+            "20",
+            "21",
+            "22",
+            "23",
+            "24",
+            "25",
+            "26",
+            "27",
+            "30",
+            "31",
+            "32",
+            "33",
+            "34",
+            "35",
+            "36",
+            "37",
+            "38",
+            "39",
+            "40",
+            "41",
+            "42",
+            "43",
+            "44",
+            "45",
+            "46",
+            "47",
+            "48",
+            "50",
+            "51",
+            "52",
+            "53",
+            "54",
+            "55",
+            "56",
+            "57",
+            "58",
+            "59",
+            "60",
+            "61",
+            "62",
+            "63",
+            "64",
+            "65",
+            "66",
+            "67",
+            "68",
+            "71",
+            "72",
+            "73",
+            "74",
+            "75",
+            "76",
+            "77",
+            "80",
+            "81",
+            "82",
+            "83",
+            "84",
+            "85",
+            "86",
+            "87",
+            "88",
+            "90",
+            "91",
+            "92",
+            "93",
+            "94",
+            "95",
+            "98",
+            "99",
+        ]
 
         ein_prefix: str = self.random_element(ein_prefix_choices)
         sequence = self.random_int(min=0, max=9999999)
 
-        ein = f'{ein_prefix:s}-{sequence:07d}'
+        ein = f"{ein_prefix:s}-{sequence:07d}"
         return ein
 
     def invalid_ssn(self) -> str:
-        """ Generate a random invalid United States Social Security Identification Number (SSN).
+        """Generate a random invalid United States Social Security Identification Number (SSN).
 
         Invalid SSNs have the following characteristics:
         Cannot begin with the number 9
@@ -180,7 +181,8 @@ class Provider(BaseProvider):
             96,
             97,
             98,
-            99]
+            99,
+        ]
         area = self.random_int(min=0, max=999)
         if area < 900 and area not in {666, 0}:
             random_group_or_serial = self.random_int(min=1, max=1000)
@@ -197,11 +199,11 @@ class Provider(BaseProvider):
             group = self.random_element([x for x in range(0, 100) if x not in itin_group_numbers])
             serial = self.random_int(0, 9999)
 
-        invalid_ssn = f'{area:03d}-{group:02d}-{serial:04d}'
+        invalid_ssn = f"{area:03d}-{group:02d}-{serial:04d}"
         return invalid_ssn
 
     def ssn(self, taxpayer_identification_number_type: str = SSN_TYPE) -> str:
-        """ Generate a random United States Taxpayer Identification Number of the specified type.
+        """Generate a random United States Taxpayer Identification Number of the specified type.
 
         If no type is specified, a US SSN is returned.
         """
@@ -225,9 +227,10 @@ class Provider(BaseProvider):
             group = self.random_int(1, 99)
             serial = self.random_int(1, 9999)
 
-            ssn = f'{area:03d}-{group:02d}-{serial:04d}'
+            ssn = f"{area:03d}-{group:02d}-{serial:04d}"
             return ssn
 
         else:
-            raise ValueError("taxpayer_identification_number_type must be one of 'SSN', 'EIN', 'ITIN',"
-                             " or 'INVALID_SSN'.")
+            raise ValueError(
+                "taxpayer_identification_number_type must be one of 'SSN', 'EIN', 'ITIN'," " or 'INVALID_SSN'."
+            )
